@@ -31,6 +31,9 @@ class BaseAPI:
     def post(self, url: str, json: dict = None):
         return self._send_request(requests.post, url, json=json)
 
+    def patch(self, url: str, json: dict = None):
+        return self._send_request(requests.patch, url, json=json)
+
     def delete(self, url: str):
         return self._send_request(requests.delete, url)
 
