@@ -28,18 +28,19 @@ def start(servlet_spec: ServletSpec, volume_request=None, persistent_dir=None, n
 
     if volume_request:
         servlet = WALServlet(servlet_spec, num_replicas, persistent_dir, volume_request=volume_request)
+        servlet.start_polling()
         logger.info(f"Before reload: {servlet_spec.get_stats()}")
         servlet.write_offset(servlet.replica, 0)  # replay all the messages
-        servlet.load_sync(sync_msg_handler)
+        loop.run_until_complete(servlet.run_sync(servlet.load_sync, sync_msg_handler))
         logger.info(f"After reload: {servlet_spec.get_stats()}")
         loop.create_task(servlet.consume_msgs())
     else:
         servlet = ZMQServlet(servlet_spec)
+        servlet.start_polling()
 
     if stats_exporter:
         loop.create_task(stats_exporter())
 
-    servlet.start_polling()
     servlet.start_reload_task()
     loop.set_exception_handler(servlet.handle_exception)
     loop.run_forever()
