@@ -3,8 +3,8 @@ from .servlet import ZMQServlet
 from pinecone.utils.constants import ZMQ_CONTROL_PORT
 from pinecone.utils import get_hostname
 from pinecone.protos import core_pb2
+from pinecone.network.zmq import SocketWrapper
 
-import zmq
 
 from typing import AsyncIterable
 
@@ -14,9 +14,9 @@ class GatewayServlet(ZMQServlet):
     async def poll_traceroute(self) -> AsyncIterable['core_pb2.TraceRoute']:
         control_sock = self.init_socket(Socket(True, SocketType.PULL, ZMQ_CONTROL_PORT, get_hostname()))
         while True:
-            yield await self.recv_traceroute(control_sock)
+            yield await self.recv_traceroute(control_sock.socket())
 
-    async def recv_traceroute(self, sock: zmq.Socket) -> 'core_pb2.TraceRoute':
+    async def recv_traceroute(self, sock: SocketWrapper) -> 'core_pb2.TraceRoute':
         msg = await sock.recv()
         msg_pb = core_pb2.TraceRoute()
         msg_pb.ParseFromString(msg)
