@@ -4,7 +4,7 @@
 from pinecone.network.zmq.function_socket import FunctionSocket
 from pinecone.utils.hash_ring import ConsistentHashRing
 
-from .exceptions import InvalidOffsetException
+from .exceptions import InvalidOffsetException, IndexBusyException
 from pinecone.protos import core_pb2
 from pinecone.network.zmq.servlet import ZMQServlet
 from pinecone.utils import open_or_create, get_hostname, wal, get_current_namespace, index_state, format_error_msg
@@ -73,6 +73,7 @@ class WALServlet(ZMQServlet):
 
         self.hash_ring = ConsistentHashRing.from_file(servlet_spec.function_name, servlet_spec.native)
         self.rebalance_requested = False
+        self.rebalance_failed = False
 
         self._rebalance_sock_spec = Socket(False, SocketType.PUSH, ZMQ_PORT_IN, self.spec.function_name)
         self.rebalance_sock = FunctionSocket(self._rebalance_sock_spec, self.context, self.spec.native)
@@ -81,6 +82,7 @@ class WALServlet(ZMQServlet):
         self.index_ready.set()
 
         self.compact_log_done = asyncio.Event()
+        self.loading_from_wal = True
 
     @property
     def log_in_socket(self):
@@ -112,28 +114,44 @@ class WALServlet(ZMQServlet):
             await self.handle_log_msg(msg)
 
     def trigger_rebalance_shards(self):
-        url = os.environ.get('CONTROLLER_URL')
-        logger.info(f"{get_hostname()}: Triggering rebalance to {url}")
+        try:
+            url = os.environ.get('CONTROLLER_URL')
+            logger.info(f"{get_hostname()}: Triggering rebalance to {url}")
 
-        project_name = get_current_namespace().replace(f"{self.spec.service_name}-", "")
-        if len(project_name) == "":
-            headers = {}
-        else:
-            # user label and user_name are placeholders, not used
-            x_user_id = ".".join(['user_label', 'user_name', project_name])
-            headers = {'x-user-id': x_user_id}
-        resp = requests.patch(f'{url}/services/{self.spec.service_name}',
-                              json={'function': {'name': self.spec.function_name, 'shards': self.hash_ring.size + 1}},
-                              headers=headers).json()
-        logger.info(resp)
-        if not resp['success']:
-            logger.error(f"Failed to trigger rebalance: {resp['msg']}. project: {project_name}. "
-                         f"service: {self.spec.service_name}. function: {self.spec.function_name}. headers: {headers}")
+            project_name = get_current_namespace().replace(f"{self.spec.service_name}-", "")
+            if len(project_name) == "":
+                headers = {}
+            else:
+                # user label and user_name are placeholders, not used
+                x_user_id = ".".join(['user_label', 'user_name', project_name])
+                headers = {'x-user-id': x_user_id}
+            resp = requests.patch(f'{url}/services/{self.spec.service_name}',
+                                  json={'function': {'name': self.spec.function_name, 'shards': self.hash_ring.size + 1}},
+                                  headers=headers).json()
+            logger.info(resp)
+            if not resp['success']:
+                logger.error(f"Failed to trigger rebalance: {resp['msg']}. project: {project_name}. "
+                             f"service: {self.spec.service_name}. function: {self.spec.function_name}. headers: {headers}")
+                self.rebalance_failed = True
+        except Exception as e:
+            logger.error(e)
+            self.rebalance_failed = True
+
+    async def index_busy(self, msg: core_pb2.Request):
+        format_error_msg(msg, IndexBusyException("Index busy reloading, try again in a minute."),
+                         self.spec.function_name)
+        await self.send_msg(msg)
 
     async def poll_sock(self, sock: SocketWrapper):
         loop = asyncio.get_event_loop()
         while True:
             msg = await self.recv_msg(sock)
+            if self.loading_from_wal:
+                await asyncio.sleep(msg.timeout)
+                if self.loading_from_wal:
+                    await self.index_busy(msg)
+                    continue
+
             if self.hash_ring:
                 msg.num_shards = self.hash_ring.size
 
@@ -176,6 +194,9 @@ class WALServlet(ZMQServlet):
                 if shard != self.spec.shard and len(msg.index.ids) > 0:
                     await self.forward_shard_msg(msg, shard)
             msg = shard_msg[self.spec.shard]
+        elif self.rebalance_requested:
+            format_error_msg(IndexBusyException("Index rebalancing, try again in a minute"), self.spec.function_name)
+            await self.send_msg(msg)
 
         new_offset = await self.put(msg)
         client_id = msg.client_id
@@ -183,7 +204,10 @@ class WALServlet(ZMQServlet):
         idx_state = await self.run_sync(index_state)
         if msg_type == 'index':
             if idx_state == IndexState.FULL:
-                format_error_msg(msg, RuntimeError("Index full, cannot accept upserts"), self.pretty_name)
+                rb_msg = "Rebalancing shards, try again in a minute." if self.rebalance_requested else ""
+                if self.rebalance_failed:
+                    rb_msg = "Scaling up shards failed. Make sure your client is up to date!"
+                format_error_msg(msg, RuntimeError(f"Index full, cannot accept upserts. {rb_msg}"), self.pretty_name)
                 await self.send_msg(msg)
                 return
             elif idx_state == IndexState.PENDING and not self.rebalance_requested:
@@ -381,12 +405,14 @@ class WALServlet(ZMQServlet):
 
     def load_sync(self, handler):
         logger.info(f'Recovering from local WAL: offset: {self.get_offset(self.replica)}')
+        self.loading_from_wal = True
         for offset, entry in self.sync_replay_log(self.get_offset(self.replica)):
             try:
                 handler(entry)
             except Exception as e:  # ignore messages that fail
                 logger.info(f'Error handling msg while recovering from WAL. offset: {offset} error: {e}')
             self.sync_ack(offset)
+        self.loading_from_wal = False
 
     def swap_log_files(self, end_file_offset: int):
         tmp_log_file = open(self.tmp_log_path, "r+b")
