@@ -5,14 +5,13 @@
 import json
 from pinecone.specs.service import Service
 from pinecone.specs.traffic_router import TrafficRouter
-from pinecone.specs import Spec
-
+from typing import Union
 mapping = {
     'Service': Service,
     'TrafficRouter': TrafficRouter
 }
 
 
-def load_json_spec(spec_json: str) -> Spec:
+def load_json_spec(spec_json: str) -> Union[Service, TrafficRouter]:
     spec = json.loads(spec_json)
     return mapping[spec['kind']].from_obj(spec)
