@@ -5,16 +5,17 @@
 from pathlib import Path
 import inspect
 
-
 from pinecone.protos import core_pb2
+from . import constants
 import numpy as np
 from loguru import logger
 import hashlib
 import os
+import re
+import traceback
 
 
-
-# import lz4.frame
+DNS_COMPATIBLE_REGEX = re.compile("^[a-z0-9]([a-z0-9]|[-])+[a-z0-9]$")
 
 
 def load_numpy(proto_arr: 'core_pb2.NdArray') -> 'np.ndarray':
@@ -79,6 +80,10 @@ def get_version():
     return Path(__file__).parent.parent.joinpath('__version__').read_text().strip()
 
 
+def get_environment():
+    return Path(__file__).parent.parent.joinpath('__environment__').read_text().strip()
+
+
 def clear_pb_repeated(field):
     while len(field) > 0:
         field.pop()
@@ -106,7 +111,10 @@ def replica_name(name: str, shard_id: int, replica_id: int = None):  # shard ==
 
 
 def replica_kube_hostname(name: str, shard_id: int, replica_id: int):
-    return replica_name(name, shard_id, replica_id) + '.' + shard_name(name, shard_id)
+    if replica_id is not None:
+        return replica_name(name, shard_id, replica_id) + '.' + shard_name(name, shard_id)
+    else:
+        return shard_name(name, shard_id)
 
 
 def replica_kube_hostname_from_shard(name: str, replica_id: int):
@@ -149,8 +157,45 @@ def get_hostname():
     return os.environ.get("HOSTNAME")
 
 
-def get_memory_usage():
+def get_container_memory_usage():
     with open('/sys/fs/cgroup/memory/memory.stat') as stat_file:
         rss_line = stat_file.readlines()[1].strip()
         return int(rss_line.split()[1])
 
+
+def validate_dns_name(name):
+    if not DNS_COMPATIBLE_REGEX.match(name):
+        raise ValueError("{} is invalid - service names and node names must consist of lower case "
+                         "alphanumeric characters or '-', start with an alphabetic character, and end with an "
+                         "alphanumeric character (e.g. 'my-name', or 'abc-123')".format(name))
+
+
+def get_current_namespace():
+    try:
+        return open("/var/run/secrets/kubernetes.io/serviceaccount/namespace").read()
+    except FileNotFoundError:
+        return None
+
+
+def index_state() -> constants.IndexState:
+    if constants.MEMORY_LIMIT_BYTES == 0:
+        return constants.IndexState.READY
+    mem_usage = get_container_memory_usage()
+    if mem_usage > constants.MEMORY_LIMIT_BYTES:
+        return constants.IndexState.FULL
+    elif mem_usage > constants.SHARD_SCALE_BYTES:
+        return constants.IndexState.PENDING
+    else:
+        return constants.IndexState.READY
+
+
+def format_error_msg(msg: core_pb2.Request, e: Exception, function_name: str):
+    msg.status.code = core_pb2.Status.StatusCode.ERROR
+    details = core_pb2.Status.Details(function=function_name,
+                                      function_id=get_hostname(),
+                                      exception=str(e),
+                                      traceback=traceback.format_exc())
+    details.time.GetCurrentTime()
+    msg.status.details.append(details)
+    logger.error(details.traceback)
+
